#!/usr/bin/env python3
"""
SPDX-License-Identifier: MIT
Copyright 2019 sysmocom s.f.m.c GmbH <info@sysmocom.de>

This is a freeswitch dialplan implementation, see:
https://freeswitch.org/confluence/display/FREESWITCH/mod_python

Find the right SIP server with mslookup (depending on the destination number)
and bridge calls accordingly.
"""
import json
import subprocess


def query_mslookup(service_type, id, id_type='msisdn'):
    query_str = '%s.%s.%s' % (service_type, id, id_type)
    print('[dialplan-dgsm] mslookup: ' + query_str)

    result_line = subprocess.check_output([
        'osmo-mslookup-client', query_str, '-f', 'json'])
    if isinstance(result_line, bytes):
        result_line = result_line.decode('ascii')

    print('[dialplan-dgsm] mslookup result: ' + result_line)
    return json.loads(result_line)


def handler(session, args):
    """ Handle calls: bridge to the SIP server found with mslookup. """
    print('[dialplan-dgsm] call handler')
    msisdn = session.getVariable('destination_number')

    # Run osmo-mslookup-client binary. We have also tried to directly call the
    # C functions with ctypes but this has lead to hard-to-debug segfaults.
    try:
        result = query_mslookup("sip.voice", msisdn)

        # This example only makes use of IPv4
        if not result['v4']:
            print('[dialplan-dgsm] no IPv4 result from mslookup')
            session.hangup('UNALLOCATED_NUMBER')
            return

        sip_ip, sip_port = result['v4']
        dial_str = 'sofia/internal/sip:{}@{}:{}'.format(
            msisdn, sip_ip, sip_port)
        print('[dialplan-dgsm] dial_str: ' + str(dial_str))

        session.execute('bridge', dial_str)
    except:
        print('[dialplan-dgsm]: exception during call handler')
        session.hangup('UNALLOCATED_NUMBER')


def fsapi(session, stream, env, args):
    """ Freeswitch refuses to load the module without this. """
    stream.write(env.serialize())


def main():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('id', type=int)
    parser.add_argument('-i', '--id-type', default='msisdn',
                        help='default: "msisdn"')
    parser.add_argument('-s', '--service', default='sip.voice',
                        help='default: "sip.voice"')
    args = parser.parse_args()

    result = query_mslookup(args.service, args.id, args.id_type)
    print(json.dumps(result))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# vim: shiftwidth=4 tabstop=4 expandtab
import subprocess
import json

def query_mslookup(query_str):
    result = {'result': 'not-found'}
    proc = subprocess.Popen(('osmo-mslookup-client', '-f', 'json', query_str),
		            stdout=subprocess.PIPE)
    for line in iter(proc.stdout.readline,''):
        if not line:
            break
        response = json.loads(line)
        if response.get('result') == 'result':
                result = response
        print('Response: %r' % response)
    return result

if __name__ == '__main__':
    import sys
    query_str = '1000-5000@sip.voice.12345.msisdn'
    if len(sys.argv) > 1:
        query_str = sys.argv[1]
    print('Final result: %r' % query_mslookup(query_str))



#!/usr/bin/env python3
"""
SPDX-License-Identifier: MIT
Copyright 2019 sysmocom s.f.m.c GmbH <info@sysmocom.de>

WARNING: this is just a proof-of-concept implementation, it blocks for every
received SMPP request and is not suitable for servicing more than one request
at a time.

Based on esme.py from RCCN (license changed with permission from author):
https://github.com/Rhizomatica/rccn/blob/master/rccn/esme.py
Copyright 2017 keith <keith@rhizomatica.org>

Forward SMS to the receiver's SMSC, as determined with mslookup.
Requires smpplip (pip3 install --user smpplib) and osmo-mslookup-client.

Example SMPP configuration for osmo-msc.cfg:
smpp
 local-tcp-ip 127.0.0.1 2775
 policy closed
 smpp-first
# outgoing to esme_dgsm.py
 esme OSMPP
  no alert-notifications
  password foo
  default-route
# incoming from esme_dgsm.py
 esme ISMPP
  no alert-notifications
  password foo
"""
import argparse
import json
import logging
import smpplib
import subprocess
import time


def can_handle_pdu(pdu):
    if not isinstance(pdu, smpplib.command.DeliverSM):
        logging.info('PDU is not a DeliverSM, ignoring')
        return False

    if int(pdu.dest_addr_ton) == smpplib.consts.SMPP_TON_INTL:
        logging.info("Unable to handle SMS for %s: SMPP_TON_INTL" %
                     (pdu.destination_addr))
        return False

    return True


def query_mslookup(service_type, id, id_type='msisdn'):
    query_str = '%s.%s.%s' % (service_type, id, id_type)
    logging.info('mslookup: ' + query_str)

    result_line = subprocess.check_output(['osmo-mslookup-client', query_str,
                                           '-f', 'json'])
    if isinstance(result_line, bytes):
        result_line = result_line.decode('ascii')

    logging.info('mslookup result: ' + result_line.rstrip())
    return json.loads(result_line)


def tx_sms(dst_host, dst_port, source, destination, registered_delivery,
           unicode_text):
    smpp_client = smpplib.client.Client(dst_host, dst_port, 90)
    smpp_client.connect()
    smpp_client.bind_transceiver(system_id=args.dst_id, password=args.dst_pass)
    logging.info('Connected to destination SMSC (%s@%s:%s)' % (args.dst_id,
                 dst_host, dst_port))

    pdu = smpp_client.send_message(
        source_addr_ton=smpplib.consts.SMPP_TON_ALNUM,
        source_addr_npi=smpplib.consts.SMPP_NPI_UNK,
        source_addr=source.decode(),
        dest_addr_ton=smpplib.consts.SMPP_TON_SBSCR,
        dest_addr_npi=smpplib.consts.SMPP_NPI_ISDN,
        destination_addr=destination.decode(),
        short_message=unicode_text,
        registered_delivery=registered_delivery,
    )

    smpp_client.unbind()
    smpp_client.disconnect()
    del pdu
    del smpp_client


def rx_deliver_sm(pdu):
    if not can_handle_pdu(pdu):
        return smpplib.consts.SMPP_ESME_RSYSERR

    msisdn = pdu.destination_addr.decode()
    logging.info("Incoming SMS for: " + msisdn)

    if args.sleep:
        logging.info("Sleeping for %i seconds" % (args.sleep))
        time.sleep(args.sleep)
        logging.info("Sleep done")

    if args.always_fail is not None:
        return args.always_fail

    result = query_mslookup("smpp.sms", msisdn)
    if 'v4' not in result or not result['v4']:
        logging.info('No IPv4 result from mslookup! This example only'
                     ' makes use of IPv4, dropping.')
        return smpplib.consts.SMPP_ESME_RSYSERR

    dst_host, dst_port = result['v4']
    tx_sms(dst_host, dst_port, pdu.source_addr,
           pdu.destination_addr, int(pdu.registered_delivery),
           pdu.short_message)

    return smpplib.consts.SMPP_ESME_ROK


def smpp_bind():
    client = smpplib.client.Client(args.src_host, args.src_port, 90)
    client.set_message_received_handler(rx_deliver_sm)
    client.connect()
    client.bind_transceiver(system_id=args.src_id, password=args.src_pass)
    logging.info('Connected to source SMSC (%s@%s:%s)' % (args.src_id,
                 args.src_host, args.src_port))
    logging.info('Waiting for SMS...')
    client.listen()


def main():
    global args
    parser = argparse.ArgumentParser()
    parser.add_argument('--src-host', default='127.0.0.1',
                        help='source SMSC (OsmoMSC) host (default: 127.0.0.1)')
    parser.add_argument('--src-port', default=2775, type=int,
                        help='source SMSC (OsmoMSC) port (default: 2775)')
    parser.add_argument('--src-id', default='OSMPP',
                        help='source system id, as configured in osmo-msc.cfg'
                             ' (default: OSMPP)')
    parser.add_argument('--src-pass', default='foo',
                        help='source system password, as configured in'
                             ' osmo-msc.cfg (default: foo)')
    parser.add_argument('--dst-id', default='ISMPP',
                        help='destination system id, as configured in'
                             ' osmo-msc.cfg (default: ISMPP)')
    parser.add_argument('--dst-pass', default='foo',
                        help='destination system password, as configured in'
                             ' osmo-msc.cfg (default: foo)')
    parser.add_argument('--sleep', default=0, type=float,
                        help='sleep time in seconds before forwarding an SMS,'
                             ' to test multithreading (default: 0)')
    parser.add_argument('--always-fail', default=None, metavar='SMPP_ESME_ERRCODE',
                        help='test delivery failure: always return an error code on Deliver-SM,'
                        ' pass an smpplib error code name like RDELIVERYFAILURE (see smpplib/consts.py),'
                        ' or an SMPP error code in hex digits')
    args = parser.parse_args()

    logging.basicConfig(level=logging.INFO, format='[%(asctime)s]'
                        ' (%(threadName)s) %(message)s', datefmt="%H:%M:%S")

    if args.always_fail:
        resolved = None
        name = 'SMPP_ESME_' + args.always_fail
        if hasattr(smpplib.consts, name):
            resolved = getattr(smpplib.consts, name)
        if resolved is None:
            try:
                resolved = int(args.always_fail, 16)
            except ValueError:
                resolved = None
        if resolved is None:
            print('Invalid argument for --always-fail: %r' % args.always_fail)
            exit(1)
        args.always_fail = resolved
        logging.info('--always-fail: returning error code %s to all Deliver-SM' % hex(args.always_fail))

    smpp_bind()


if __name__ == "__main__":
    main()

# vim: expandtab tabstop=4 shiftwidth=4



#!/usr/bin/env python3
# vim: shiftwidth=4 tabstop=4 expandtab
import socket
import time

MSLOOKUP_SOCKET_PATH = '/tmp/mslookup'

def query_mslookup_socket(query_str, socket_path=MSLOOKUP_SOCKET_PATH):
    mslookup_socket = socket.socket(socket.AF_UNIX, socket.SOCK_SEQPACKET)
    mslookup_socket.setblocking(True)
    mslookup_socket.connect(socket_path)
    result = {'result': 'not-found'}
    column_names = mslookup_socket.recv(1024).decode('ascii')
    if not column_names:
        return result
    column_names = column_names.split('\t')
    mslookup_socket.sendall(query_str.encode('ascii'))
    while True:
        csv = mslookup_socket.recv(1024).decode('ascii')
        if not csv:
            break
        response = dict(zip(column_names, csv.split('\t')))
        if response.get('result') == 'result':
            result = response
        print('Response: %r' % response)
    return result

if __name__ == '__main__':
    import sys
    print(
        '\nPlease run separately: osmo-mslookup-client --socket /tmp/mslookup -d\n')
    query_str = '1000-5000@sip.voice.12345.msisdn'
    if len(sys.argv) > 1:
        query_str = sys.argv[1]
    print('Final result: %r' % query_mslookup_socket(query_str))



#!/usr/bin/env python3
# Convert test sets pasted from 3GPP TS 55.205 to C code.
 
# (C) 2016 by sysmocom s.f.m.c. GmbH <info@sysmocom.de>
#
# All Rights Reserved
#
# Author: Neels Hofmeyr <nhofmeyr@sysmocom.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os

script_dir = sys.path[0]

fields = (
  'Ki',
  'RAND',
  'OP',
  'OPc',
  'MIL3G-RES',
  'SRES#1',
  'SRES#2',
  'MIL3G-CK',
  'MIL3G-IK',
  'Kc',
)

test_sets_lines = []
test_set_lines = None

for line in [l.strip() for l in open(os.path.join(script_dir, 'ts55_205_test_sets.txt'), 'r')]:
  if line.startswith('Test Set'):
    if test_set_lines:
      test_sets_lines.append(test_set_lines)
    test_set_lines = []
  elif len(line) == 8:
    try:
      is_hex = int(line, 16)
      test_set_lines.append(line)
    except ValueError:
      pass

if test_set_lines:
  test_sets_lines.append(test_set_lines)

# Magic fixups for PDF-to-text uselessness
idx = (( 0, 10, 15, 19),
       ( 1, 11, 16, 20),
       ( 2, 12, 17, 21),
       ( 3, 13, 18, 22),
       ( 4, 14),
       ( 5, ),
       ( 6, ),
       ( 7, 23, 26, 28),
       ( 8, 24, 27, 29),
       ( 9, 25 ),
      )

test_sets = []
for l in test_sets_lines:
  test_sets.append( [ ''.join([l[i] for i in li]) for li in idx ] )

func_templ = open(os.path.join(script_dir, 'func_template.c'), 'r').read()

funcs = []
func_calls = []
nr = 0
for test_set in test_sets:
  nr += 1
  func_name = 'test_set_%d' % nr
  kwargs = dict(zip(fields, test_set))
  kwargs['func_name'] = func_name

  func_calls.append('\t%s();' % func_name)
  funcs.append(func_templ.format(**kwargs))

templ = open(os.path.join(script_dir, 'main_template.c')).read()

code = templ.replace('FUNCTIONS', '\n'.join(funcs)).replace('FUNCTION_CALLS', '\n'.join(func_calls))

print('''
/***** DO NOT EDIT THIS FILE -- THIS CODE IS GENERATED *****
 ***** by gen_ts_55_205_test_sets/pdftxt_2_c.py        *****/
''')
print(code)

